{% load static sekizai_tags sass_tags %}

{% addtoblock "js" %}<script src="{% static 'shop/js/auth.js' %}" type="text/javascript"></script>{% endaddtoblock %}
{% addtoblock "ng-requires" %}django.shop.auth{% endaddtoblock %}

{% if customer.is_authenticated %}
<div class="panel panel-info">
	<div class="panel-heading">
		<div class="panel-title">{% block form_title %}Panel Title{% endblock %}</div>
	</div>

	<div class="panel-body">
		<form shop-auth-form action="{{ action }}" role="form">
	{% block form_content %}
		<div class="alert alert-success" ng-show="success_message" ng-cloak>
			<a class="close" ng-click="dismiss()">×</a>{% verbatim %}{{ success_message }}{% endverbatim %}
		</div>
		<div class="alert alert-danger" ng-show="error_message" ng-cloak>
			<a class="close" ng-click="dismiss()">×</a>{% verbatim %}{{ error_message }}{% endverbatim %}
		</div>
	{% endblock form_content %}
		</form>
	</div>
</div>
{% endif %}
