{% load i18n static sekizai_tags sass_tags %}

{% addtoblock "css" %}<link href="{% sass_src 'shop/css/product-list.scss' %}" rel="stylesheet" type="text/css" />{% endaddtoblock %}

{% addtoblock "js" %}<script src="{% static 'shop/js/utils.js' %}" type="text/javascript"></script>{% endaddtoblock %}
{% addtoblock "js" %}<script src="{% static 'shop/js/catalog.js' %}" type="text/javascript"></script>{% endaddtoblock %}
{% addtoblock "ng-requires" %}django.shop.catalog{% endaddtoblock %}

<shop-catalog-list>
	<div class="row" ng-if="catalog.count===undefined">
	{% for product in data.results %}
		{% include "shop/catalog/product-list-item.html" with ng=0 %}
	{% endfor %}
	</div>
	<div class="row" ng-cloak ng-if="catalog.count>=0">
		{% include "shop/catalog/product-list-item.html" with ng=1 %}
	</div>
</shop-catalog-list>

{% include "shop/catalog/paginator.html" %}
