{% extends "shop/auth/authenticated-base.html" %}
{% load i18n %}

{% block form_title %}{% trans "Change Password" %}{% endblock %}

{% block form_content %}
	{{ block.super }}
<label class="control-label">{% trans "New Password" %}</label>
<div style="margin-bottom: 25px" class="input-group">
	<span class="input-group-addon"><i class="fa fa-key fa-fw"></i></span>
	<input type="password" class="form-control" name="new_password1" ng-model="form_data.new_password1" placeholder="{% trans 'New Password' %}" />
</div>

<label class="control-label">{% trans "Confirm Password" %}</label>
<div style="margin-bottom: 25px" class="input-group">
	<span class="input-group-addon"><i class="fa fa-key fa-fw"></i></span>
	<input type="password" class="form-control" name="new_password2" ng-model="form_data.new_password2" placeholder="{% trans 'Confirm Password' %}" />
</div>

<div class="form-group">
	<button type="button" ng-click="submitForm('{% url 'shop:password-change' %}', 5000)" class="btn btn-primary">{% trans "Submit Changes" %}</button>
</div>
{% endblock form_content %}
