<div class="form-group">
	<h4>{{ shipping_method_form.shipping_modifier.label }}</h4>
	<pre class="shop-checkout-summary" djng-bind-if="checkout_summary.shipping_method_form">{{ shipping_method_form.as_text }}</pre>
</div>