{% extends "shop/auth/anonymous-base.html" %}
{% load i18n %}

{% block form_title %}{% trans "Password Forgotten?" %}{% endblock %}

{% block form_content %}
	{{ block.super }}
<label class="control-label" ng-hide="success_message">{% trans "Your e-mail address" %}</label>
<div style="margin-bottom: 20px" class="input-group" ng-hide="success_message">
	<span class="input-group-addon"><i class="fa fa-envelope-o fa-fw"></i></span>
	<input type="text" class="form-control" name="email" ng-model="form_data.email" value="" placeholder="{% trans 'Email' %}" />
</div>

<div class="form-group" ng-hide="success_message">
	<button type="button" ng-click="submitForm('{% url 'shop:password-reset' %}', 5000)" class="btn btn-primary">{% trans "Reset Password" %}</button>
</div>
{% endblock %}
